
//=======================================================
//  This code is generated by Terasic System Builder
//=======================================================

//THIS PROJECT DEVELOPED BY CORNELL ECE 3400 STAFF. PLEASE
//DO NOT REUSE OR DISTRIBUTE THIS CODE WITHOUT PERMISSION
//SPRING 2015

module VGAlogic(
 
	//////////// CLOCK //////////
	CLOCK_50,

	//////////// KEY //////////
	// KEY 0 inits the screen
	// KEY 1 writes individual blocks
	KEY,

	////// GPIO_0, GPIO_0 connect to GPIO Default //////
	GPIO,
	GPIO_IN 
);


//=======================================================
//  PORT declarations
//=======================================================

//////////// CLOCK //////////
input 		          		CLOCK_50;

//////////// KEY //////////
input 		     [1:0]		KEY;

///// GPIO_0, GPIO_0 connect to GPIO Default //////////
inout 		    [33:0]		GPIO;
input 		     [1:0]		GPIO_IN;


//=======================================================
//  REG/WIRE declarations
//=======================================================
reg CLOCK_25;
wire [9:0] PIXEL_COORD_X;
wire [9:0] PIXEL_COORD_Y;
wire [7:0] PIXEL_COLOR;

wire reset;
reg [7:0] pixel_in;
wire [14:0] pixel_r_index;
reg [14:0] pixel_w_index;
wire [7:0] pixel_out;
reg pixel_wen;

reg [14:0] counter; //state variable to draw screen
reg counter_go; //counter enable
reg [31:0] timer ; // slow screen update

//draw command state variable
reg key_1_down ;

VGADriver driver(
	.CLOCK(CLOCK_25),
	.PIXEL_COLOR_IN(pixel_out),
	.PIXEL_X(PIXEL_COORD_X),
	.PIXEL_Y(PIXEL_COORD_Y),
	.PIXEL_COLOR_OUT({GPIO[9],GPIO[11],GPIO[13],GPIO[15],GPIO[17],GPIO[19],GPIO[21],GPIO[23]}),
	.H_SYNC_NEG(GPIO[7]),
	.V_SYNC_NEG(GPIO[5])
);

blockArray pixelArray(
	.clk(CLOCK_50),
	.r_index(pixel_r_index),
	.w_index(pixel_w_index),
	.value(pixel_in),
	.w_en(pixel_wen),
	.out(pixel_out)
);

// map pixel coods from VGA driver to memory address
assign pixel_r_index = {PIXEL_COORD_X[9:2], PIXEL_COORD_Y[8:2] };

// make 4x4 blocks by truncaing low order 2 bits
wire [9:0] line_1_y, line_2_x ;
// positions of two lines in 640x480 cooridinates
assign line_1_y = 10'd240;
assign line_2_x = 10'd320;

//=======================================================
//  Structural coding
//=======================================================
always @(posedge CLOCK_50) begin
	CLOCK_25 <= ~CLOCK_25; //VGA needs 25 MHz clock - FPGA has 50 MHz clock
	timer <= timer + 1;
	if (reset) begin
		counter <= 0; //iterate through all memory locations
		counter_go <= 1;
	end
	
	// after reset draw the screen
	else if (counter_go) begin
		pixel_wen <= 1; //write 
		//location is x, 7 bits of y=240>>2 7'b0111100
		// scale the 640x480 x,y by 4
		pixel_w_index <= {counter[9:2], line_1_y[8:2]}; 
		// make a nice red to white color bar in 8 bit color
		pixel_in <= {counter[4:2],counter[7:5], counter[9:8]}; 
		
	// detect end of reset drawing	
		if (counter == 15'd639) begin
			counter_go <= 0; //end of loop
			pixel_wen <= 0;
			counter <= 15'b0 ;
		end
		else counter <= counter + 15'd1;
	end
	
	// draw a single square
	if (~KEY[1] && ~key_1_down) begin // negative edge of KEY 1
		key_1_down <= 1'b1;
		counter <= counter + 15'd4;
		pixel_in <= 8'b111_111_11;
		//location 8 bits of x=320>>2, y ; 8'b01010000
		// scale the 640x480 x,y by 4
		pixel_w_index <= {line_2_x[9:2], counter[8:2] }; 
		pixel_wen <= 1'b1 ;
	end
	// turn off write after one cycle
	else if (~KEY[1] && pixel_wen) begin
		pixel_wen <= 1'b0 ;
	end
	// detect key release
	else if (KEY[1] && key_1_down) begin
		key_1_down <= 1'b0;
		pixel_wen <= 1'b0 ;
	end	
end

// the state machine reset
assign reset = ~KEY[0];

endmodule
